# coding: utf-8
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel
from jmb.address.models import AddressCoordinates


# http://www.istat.it/it/archivio/6789


class Region(DateModel, UserModel):
    """
    Region model
    """
    code = models.CharField(
        max_length=10,
        unique=True,
        verbose_name=_('code'),
        primary_key=True,
    )
    name = models.CharField(
        max_length=100,
        unique=True,
        verbose_name=_('name'),
    )

    class Meta(object):
        verbose_name = _('region')
        verbose_name_plural = _('regions')
        ordering = ('name',)

    def __unicode__(self):
        return self.name

    def localities(self):
        Locality.objects.filter(province__region=self)

    @staticmethod
    def get_choices():
        region_list = []
        for region in Region.objects.all().order_by('name'):
            region_list.append((region.code, region.name))
        return region_list


class Province(DateModel, UserModel):
    """
    Italian Province Model
    """
    code = models.CharField(
        max_length=10,
        unique=True,
        verbose_name=_('code'),
        primary_key=True,
    )
    region = models.ForeignKey(
        Region,
        blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name="provinces",
        verbose_name=_('region'),
    )
    name = models.CharField(
        max_length=100,
        unique=True,
        verbose_name=_('name'),
    )
    abbr = models.CharField(
        max_length=2,
        unique=True,
        verbose_name=_('abbreviation'),
    )

    class Meta(object):
        verbose_name = _('province')
        verbose_name_plural = _('provinces')
        ordering = ('name',)

    def __unicode__(self):
        return self.name

    @staticmethod
    def get_choices():
        province_list = []
        for province in Province.objects.all().order_by('name'):
            province_list.append((province.code, province.name))
        return province_list


class Locality(DateModel, UserModel):
    """
    Italian Locality Model
    """
    istat_code = models.CharField(
        max_length=10,
        unique=True,
        verbose_name=_('ISTAT code'),
        primary_key=True,
    )
    code = models.CharField(
        max_length=10,
        verbose_name=_('code'),
    )
    province = models.ForeignKey(
        Province,
        blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name="localities",
        verbose_name=_('province'),
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )

    def get_region(self):
        return self.province.region if self.province else _('NOT SET')
    get_region.short_description = _('region')

    class Meta(object):
        verbose_name = _('locality')
        verbose_name_plural = _('localities')
        ordering = ('name',)

    def __unicode__(self):
        return self.name

    @staticmethod
    def get_choices():
        locality_list = []
        for locality in Locality.objects.all().order_by('name'):
            locality_list.append((locality.code, locality.name))
        return locality_list


class ZipCode(DateModel, UserModel):
    # pk_code = models.CharField(
    #     max_length=20,
    #     unique=True,
    #     verbose_name=_('pk code'),
    #     primary_key=True,
    # )
    code = models.CharField(
        max_length=5,
        verbose_name=_('code'),
    )
    locality = models.ManyToManyField(
        Locality,
        # blank=True, null=True,
        # on_delete=models.SET_NULL,
        related_name="caps",
        verbose_name=_('locality'),
    )

    class Meta(object):
        verbose_name = _('zip code')
        verbose_name_plural = _('zip codes')
        ordering = ('code',)
        # unique_together = ('code', 'locality',)

    def __unicode__(self):
        return u"{}".format(self.code)

    # def save(self, *args, **kwargs):
    #     locality_istat_code = self.locality.istat_code if self.locality else _('NOT SET')
    #     self.pk_code = "{0}_{1}".format(self.code, locality_istat_code)
    #     super(ZipCode, self).save(*args, **kwargs)

    def get_region(self):
        return self.locality.all()[0].get_region() if len(self.locality.all()) > 0 else _('NOT SET')
    get_region.short_description = _('region')

    def get_province(self):
        return self.locality.all()[0].province if len(self.locality.all()) > 0 else _('NOT SET')
    get_province.short_description = _('province')

    @staticmethod
    def get_choices():
        zip_list = []
        for zip_code in ZipCode.objects.all().order_by('code'):
            zip_list.append((zip_code.code, unicode(zip_code)))
        return zip_list


class Address(AddressCoordinates):
    it_name = models.CharField(
        blank=True, null=True,
        max_length=100,
        db_index=True,
        verbose_name=_("address or place"),
    )
    it_number = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("number"),
        db_column="it_anumber"
    )
    it_region = models.ForeignKey(
        Region,
        related_name='addresses',
        verbose_name=_("region"),
        blank=True, null=True,
    )
    it_province = models.ForeignKey(
        Province,
        related_name='addresses',
        verbose_name=_("province"),
        blank=True, null=True,
    )
    it_locality = models.ForeignKey(
        Locality,
        related_name='addresses',
        verbose_name=_("locality"),
        blank=True, null=True,
    )
    it_zip_code = models.ForeignKey(
        ZipCode,
        related_name='addresses',
        verbose_name=_("zip code"),
        blank=True, null=True,
    )

    geocoding_fields = ['it_name', 'it_region', 'it_province', 'it_locality', 'it_zip_code', ]

    class Meta:
        abstract = True
        verbose_name = _('address')
        verbose_name_plural = _('addresses')

    def __unicode__(self):
        pass #TODO
        # return u"{address} {number}, {cap} {city} ({prov}) - {country}".format(
        #     address=self.name or '',
        #     number=self.number or '',
        #     cap=self.zip_code or '',
        #     city=self.locality or '',
        #     prov=self.province or '',
        #     country=self.country or '',
        # )
